"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PagesBuilder = exports.pagesStorage = void 0;

var Event = require("events");

var chunk = require("chunk");

var _vkIo = require("vk-io");

var _ContextUtils = require("./ContextUtils");

var _functions = require("./functions");

// @ts-ignore
const pagesStorage = new Map();
exports.pagesStorage = pagesStorage;

class PagesBuilder extends Event.EventEmitter {
  constructor({
    context
  }) {
    super();
    this.id = (0, _functions.randomString)(6);
    this.pages = [];
    this.header = "";
    this.footer = "";
    this.currentPage = 1;
    this.infinityLoop = true;
    this.resetTimeout = false;
    this.pagesNumberFormat = "%c / %m";
    this.sendMethod = "send_new";
    this.listenTime = 5 * 60 * 1000; // 5 минут

    this._listenTimeout = null;
    this.listenUsers = [];
    this.triggers = new Map();
    this.sentContext = null;
    this.keyboard = _vkIo.Keyboard.builder().inline();
    this._context = context;
    this.setDefaultButtons();
  }
  /*
   * Метод для начальной установки страниц
   */


  setPages(pages) {
    this.pages = !Array.isArray(pages) ? [pages] : pages;
    return this;
  }
  /*
   * Метод для добавления страниц в конец
   */


  addPages(pages) {
    this.pages = this.pages.concat(pages);
    return this;
  }
  /*
   * Метод для автоматической генерации страниц
   */


  autoGeneratePages({
    items,
    countPerPage = 10
  }) {
    const chunks = chunk(items, countPerPage);
    this.setPages(chunks.map(chunk => ({
      message: chunk.join("\n")
    })));
    return this;
  }
  /*
   * Метод для открытия определенной страницы
   */


  async setPage(pageNumber = this.currentPage) {
    this.currentPage = pageNumber;

    if (this.resetTimeout) {
      this.resetListenTimeout();
    }

    this.saveContext();
    const page = await this.getPage(pageNumber);
    this.emit("page_set", pageNumber);
    return new _ContextUtils.ContextUtils(this).editMessage(page);
  }
  /*
   * Метод для установки автоматического сброса таймера при переключении между страницами
   */


  autoResetTimeout(status = true) {
    this.resetTimeout = status;
    this.saveContext();
    return this;
  }
  /*
   * Метод для установки формата нумерования страниц
   */


  setPagesNumberFormat(format = "%c / %m") {
    this.pagesNumberFormat = format;
    this.saveContext();
    return this;
  }
  /*
   * Метод для установки бесконечного переключения между страницами при достижении конца
   */


  setInfinityLoop(status = true) {
    this.infinityLoop = status;
    this.saveContext();
    return this;
  }
  /*
   * Метод для установки верхней части страниц
   */


  setPagesHeader(header = "") {
    this.header = header;
    this.saveContext();
    return this;
  }
  /*
   * Метод для установки нижней части страниц
   */


  setPagesFooter(footer = "") {
    this.footer = footer;
    this.saveContext();
    return this;
  }
  /*
   * Метод для установки метода отправки страницы
   */


  setSendMethod(method = "send_new") {
    this.sendMethod = method;
    this.saveContext();
    return this;
  }

  async getPage(pageNumber = this.currentPage) {
    var _page$keyboard;

    let page = this.pages[pageNumber - 1];

    if (typeof page === "function") {
      page = await page(this._context);
    }

    if (typeof page === "string") {
      page = {
        message: page
      };
    }

    let keyboard = (_page$keyboard = page.keyboard) !== null && _page$keyboard !== void 0 ? _page$keyboard : this.keyboard;

    if (typeof keyboard === "function") {
      keyboard = await keyboard(this._context);
    }

    if (!this.infinityLoop) {
      keyboard = this.cleanUpKeyboard(keyboard);
    } else {
      keyboard = JSON.parse(keyboard.toString());
    }

    const pagePagination = this.pagesNumberFormat.replace("%c", String(pageNumber)).replace("%m", String(this.pages.length));
    return { ...page,
      message: `${this.header}${page.message}${this.footer}\n\n${pagePagination}`,
      keyboard: JSON.stringify(keyboard.buttons.length ? keyboard : {})
    };
  }
  /*
   * Метод для установки времени прослушивания обновлений для переключения страниц
   */


  setListenTime(time = 5 * 60 * 1000) {
    this.listenTime = time;
    this.saveContext();
    return this;
  }
  /*
   * Метод для сброса текущего таймера прослушивания
   */


  resetListenTimeout({
    isFirstBuild = false
  } = {}) {
    if (this._listenTimeout || isFirstBuild) {
      clearTimeout(this._listenTimeout);
      this._listenTimeout = setTimeout(this.stopListen.bind(this), this.listenTime);
      this.saveContext();
      this.emit("listen_reset_timeout");
    }
  }
  /*
   * Метод для установки прослушивания определенных пользователей
   */


  setListenUsers(users = []) {
    if (!Array.isArray(users)) {
      users = [users];
    }

    this.listenUsers = users;
    this.saveContext();
    return this;
  }
  /*
   * Метод для добавления прослушивания определенных пользователей
   */


  addListenUsers(users = []) {
    this.listenUsers = this.listenUsers.concat(users);
    this.saveContext();
    return this;
  }
  /*
   * Метод для досрочной остановки прослушивания новых сообщений
   */


  async stopListen() {
    if (this._listenTimeout) {
      clearTimeout(this._listenTimeout);
      pagesStorage.delete(this.id);
      this.emit("listen_stop", this.currentPage);
      const page = await this.getPage(this.currentPage);
      new _ContextUtils.ContextUtils(this).editMessage({ ...page,
        keyboard: JSON.stringify({})
      }, "stop");
    }
  }
  /*
   * Метод для установки кнопок по умолчанию
   */


  setDefaultButtons({
    buttons = ["first", "back", "stop", "next", "last"],
    type = "text"
  } = {}) {
    const keyboard = _vkIo.Keyboard.builder().inline(true);

    const defaultButtons = new Map([["first", "⏪"], ["back", "◀️"], ["stop", "⏹"], ["next", "▶️"], ["last", "⏩"]]);
    buttons.forEach(button => {
      const label = defaultButtons.get(button);

      if (label) {
        const buttonObject = {
          label,
          payload: {
            builder_action: button,
            builder_id: this.id
          }
        };

        switch (type) {
          case "callback":
            keyboard.callbackButton(buttonObject);
            break;

          default:
            keyboard.textButton(buttonObject);
        }
      }
    });
    keyboard.row();
    this.keyboard = keyboard;
    this.saveContext();
    return this;
  }
  /*
   * Метод для обновления клавиатуры
   */


  updateKeyboard(keyboard = null) {
    this.keyboard = keyboard !== null && keyboard !== void 0 ? keyboard : _vkIo.Keyboard.builder().inline(true);
    this.saveContext();
    return this;
  }
  /*
   * Метод для начальной установки триггеров
   */


  setTriggers(triggers) {
    if (!Array.isArray(triggers)) {
      triggers = [triggers];
    }

    this.triggers = new Map(triggers.map(({
      name,
      callback
    }) => [name, callback]));
    this.saveContext();
    return this;
  }
  /*
   * Метод для добавления триггеров
   */


  addTriggers(triggers) {
    if (!Array.isArray(triggers)) {
      triggers = [triggers];
    }

    triggers.forEach(({
      name,
      callback
    }) => {
      this.triggers.set(name, callback);
    });
    this.saveContext();
    return this;
  }

  executeTrigger(trigger) {
    const triggerAction = this.triggers.get(trigger);

    if (triggerAction) {
      this.emit("trigger_execute", trigger);
      triggerAction(this._context);
    }
  }
  /*
   * Метод для сборки и отправки страниц
   */


  build(loader = null) {
    const {
      _context: context,
      pages
    } = this; // eslint-disable-next-line no-async-promise-executor

    return new Promise(async (resolve, reject) => {
      if (pages.length === 0) {
        return reject(new Error("Pages not set"));
      }

      if (loader !== null && loader !== void 0 && loader.id) {
        this.sentContext = loader;

        if (this.sendMethod === "send_new") {
          loader.deleteMessage({
            delete_for_all: 1
          });
        }
      }

      const page = await this.getPage(1);
      (loader ? new _ContextUtils.ContextUtils(this).editMessage(page) : context.send(page)).then(() => {
        this.emit("listen_start");
        this.resetListenTimeout({
          isFirstBuild: true
        });
        resolve(this);
      }).catch(reject);
    });
  }

  saveContext() {
    pagesStorage.set(this.id, this);
  }

  cleanUpKeyboard(keyboardBuilder) {
    const keyboard = JSON.parse(keyboardBuilder.toString());
    keyboard.buttons = keyboard.buttons.map(row => row.filter(button => {
      try {
        const payload = JSON.parse(button.action.payload);

        if (Object.keys(payload).filter(key => key !== "builder_action" && key !== "builder_id").length > 0) {
          return true;
        }

        const {
          builder_action
        } = payload;

        if (builder_action) {
          return !(this.currentPage === 1 && (builder_action === "first" || builder_action === "back") || this.currentPage === this.pages.length && (builder_action === "last" || builder_action === "next"));
        } // eslint-disable-next-line no-empty

      } catch {}

      return true;
    })).filter(row => row.length);
    return keyboard;
  }

  executeAction(action) {
    this.emit("page_action_execute", action);

    switch (action) {
      case "first":
        if (this.currentPage === 1) {
          if (this.infinityLoop) {
            this.setPage(this.pages.length);
          }

          return;
        }

        this.setPage(1);
        break;

      case "back":
        if (this.currentPage === 1) {
          if (this.infinityLoop) {
            this.setPage(this.pages.length);
          }

          return;
        }

        this.setPage(this.currentPage - 1);
        break;

      case "stop":
        this.stopListen();
        break;

      case "next":
        if (this.currentPage === this.pages.length) {
          if (this.infinityLoop) {
            this.setPage(1);
          }

          return;
        }

        this.setPage(this.currentPage + 1);
        break;

      case "last":
        if (this.currentPage === this.pages.length) {
          if (this.infinityLoop) {
            this.setPage(1);
          }

          return;
        }

        this.setPage(this.pages.length);
        break;
    }
  }

  messageMiddleware(context) {
    this._context = context;
    const payload = (context === null || context === void 0 ? void 0 : context.messagePayload) || (context === null || context === void 0 ? void 0 : context.eventPayload);
    const action = payload === null || payload === void 0 ? void 0 : payload.builder_action;
    const builderId = payload === null || payload === void 0 ? void 0 : payload.builder_id;
    const page = Number(payload === null || payload === void 0 ? void 0 : payload.builder_page);
    const trigger = payload === null || payload === void 0 ? void 0 : payload.builder_trigger;

    if ((action || page || trigger) && builderId === this.id) {
      if (this.listenUsers.length && !this.listenUsers.includes(context.senderId)) {
        return;
      }

      const contextUtils = new _ContextUtils.ContextUtils(this);

      if (context.type !== "message_event") {
        contextUtils.markAsRead(context);
      }

      if (action) {
        this.executeAction(action);
      } else if (page) {
        if (page >= 1 && page <= this.pages.length) {
          if (page === this.currentPage && context.type !== "message_event") {
            return;
          }

          this.setPage(page);
        }
      }

      if (trigger) {
        this.executeTrigger(trigger);
      }
    }
  }

}

exports.PagesBuilder = PagesBuilder;